@@ -19,19 +19,22 @@ find_dependencies - returns a list of dependencies for a given script and
 from __future__ import with_statement
 import os
 import sys
+import warnings
 
-from sumatra.dependency_finder import core, neuron, python
+from sumatra.dependency_finder import core, neuron, python, genesis
     
 
-def find_dependencies(filename, executable, on_changed='error'):
+def find_dependencies(filename, executable):
     """Return a list of dependencies for a given script and programming
        language."""
     if executable.name == "Python":
-        return python.find_dependencies(filename, on_changed)
+        return python.find_dependencies(filename, executable.path)
     elif executable.name == "NEURON":
-        return neuron.find_dependencies(filename, on_changed)
+        return neuron.find_dependencies(filename, executable.path)
+    elif executable.name == "GENESIS":
+        return genesis.find_dependencies(filename, executable.path)
     else:
-        raise Exception("find_dependencies() not yet implemented for %s" % executable.name)
-
+        warnings.warn("find_dependencies() not yet implemented for %s" % executable.name)
+        return []
         
 
