@@ -19,7 +19,7 @@ get_formatter() - return an approriate Formatter object for a given requested
 import textwrap
 
 fields = ['label', 'reason', 'outcome', 'duration', 'repository', 'main_file',
-          'version', 'executable', 'timestamp', 'tags']
+          'version', 'script_arguments', 'executable', 'timestamp', 'tags']
 
 class Formatter(object):
     
@@ -35,8 +35,7 @@ class TextFormatter(Formatter):
     def short(self):
         return "\n".join(record.label for record in self.records)
             
-    def long(self):
-        text_width = 80; indent = 13
+    def long(self, text_width=80, indent=17):
         output = ""
         for record in self.records:
             output += "-" * text_width + "\n"
@@ -123,18 +122,20 @@ class TextDiffFormatter(Formatter):
             return x and "yes" or "no"
         D = self.diff
         output = textwrap.dedent("""\
-            Record 1              : %s
-            Record 2              : %s
-            Executable differs    : %s
-            Code differs          : %s
-              Repository differs  : %s
-              Main file differs   : %s
-              Version differs     : %s
-              Non checked-in code : %s
-              Dependencies differ : %s 
-            Launch mode differs   : %s
-            Parameters differ     : %s
-            Data differs          : %s""" % (
+            Record 1                : %s
+            Record 2                : %s
+            Executable differs      : %s
+            Code differs            : %s
+              Repository differs    : %s
+              Main file differs     : %s
+              Version differs       : %s
+              Non checked-in code   : %s
+              Dependencies differ   : %s 
+            Launch mode differs     : %s
+            Input data differ       : %s
+            Script arguments differ : %s
+            Parameters differ       : %s
+            Data differ             : %s""" % (
                 D.recordA.label,
                 D.recordB.label,
                 yn(D.executable_differs),
@@ -143,6 +144,8 @@ class TextDiffFormatter(Formatter):
                 yn(D.version_differs), yn(D.diff_differs),
                 yn(D.dependencies_differ),
                 yn(D.launch_mode_differs),
+                yn(D.input_data_differ),
+                yn(D.script_arguments_differ),
                 yn(D.parameters_differ),
                 yn(D.data_differs))
             )
@@ -150,6 +153,17 @@ class TextDiffFormatter(Formatter):
     
     def long(self):
         output = ''
+        if self.diff.executable_differs:
+            output += "Executable differences:\n"
+            for record in (self.diff.recordA, self.diff.recordB):
+                output += "  %s: %s\n" % (record.label, record.executable)
+        if self.diff.code_differs:
+            output += "Code differences:\n"
+            for record in (self.diff.recordA, self.diff.recordB):
+                output += "  %s: main file '%s' at version %s in %s\n" % (record.label, record.main_file, record.version, record.repository)
+            if self.diff.diff_differs:
+                for record in (self.diff.recordA, self.diff.recordB):
+                    output += "  %s:\n %s\n" % (record.label, record.diff)
         if self.diff.dependencies_differ:
             diffs = self.diff.dependency_differences
             output += "Dependency differences:\n"
@@ -161,9 +175,9 @@ class TextDiffFormatter(Formatter):
                     output += "    B: version=%s\n" % depB.version
                     output += "       %s\n" % depA.diff.replace("\n", "\n       ")
                 elif depB is None:
-                    output += "  %s is a dependency of %s but not of %s" % (name, self.diff.recordA.label, self.diff.recordB.label)
+                    output += "  %s is a dependency of %s but not of %s\n" % (name, self.diff.recordA.label, self.diff.recordB.label)
                 elif depA is None:
-                    output += "  %s is a dependency of %s but not of %s" % (name, self.diff.recordB.label, self.diff.recordA.label)
+                    output += "  %s is a dependency of %s but not of %s\n" % (name, self.diff.recordB.label, self.diff.recordA.label)
                     
         diffs = self.diff.launch_mode_differences
         if diffs:
@@ -171,7 +185,18 @@ class TextDiffFormatter(Formatter):
             modeA, modeB = diffs
             output += "  %s: %s\n" % (self.diff.recordA.label, modeA)
             output += "  %s: %s\n" % (self.diff.recordB.label, modeB)
-
+        if self.diff.parameters_differ:
+            output += "Parameter differences:\n"
+            for record in (self.diff.recordA, self.diff.recordB):
+                output += "  %s:\n%s\n" % (record.label, record.parameters)
+        if self.diff.input_data_differ:
+            output += "Input data differences:\n"
+            for record in (self.diff.recordA, self.diff.recordB):
+                output += "  %s: %s\n" % (record.label, record.input_data)
+        if self.diff.script_arguments_differ:
+            output += "Script argument differences:\n"
+            for record in (self.diff.recordA, self.diff.recordB):
+                output += "  %s: %s\n" % (record.label, record.script_arguments)
         diffs = self.diff.data_differences
         if diffs:
             output += "Output data differences:\n"
@@ -184,7 +209,6 @@ class TextDiffFormatter(Formatter):
                     output += "  %s is generated by %s but not by %s" % (name, self.diff.recordA.label, self.diff.recordB.label)
                 elif depA is None:
                     output += "  %s is generated by %s but not by %s" % (name, self.diff.recordB.label, self.diff.recordA.label)
-        # to be completed...
         return output
     
 formatters = {
