@@ -16,6 +16,7 @@ import subprocess
 import os
 from sumatra.programs import Executable
 import warnings
+import cmd
 
 class PlatformInformation(object):
     """
@@ -36,20 +37,16 @@ class PlatformInformation(object):
     # get_info('blas_opt')
 
 
-def check_files_exist(executable, main_file, parameter_file):
+def check_files_exist(*paths):
     """
     Check that the given paths exist and return the list of paths.
     
     parameter_file may be None, in which case it is not included in the list of
     paths.
     """
-    paths = [executable.path, main_file]
-    if parameter_file:
-        paths.append(parameter_file)
     for path in paths:
         if not os.path.exists(path):
             raise IOError("%s does not exist." % path)
-    return paths
 
 
 class LaunchMode(object):
@@ -57,7 +54,7 @@ class LaunchMode(object):
     Base class for launch modes (serial, distributed, batch, ...)
     """
     
-    def get_state(self):
+    def __getstate__(self):
         """
         Since each subclass has different attributes, we provide this method
         as a standard way of obtaining these attributes, for database storage,
@@ -67,24 +64,25 @@ class LaunchMode(object):
     
     def pre_run(self, executable):
         """Run tasks before the simulation/analysis proper.""" # e.g. nrnivmodl
-        #should get the tasks to run from the Executable 
-        pass
+        # this implementation is a temporary hack. "pre_run" should probably be an Executable instance, not a string
+        if hasattr(executable, "pre_run"):
+            p = subprocess.Popen(executable.pre_run, shell=True, stdout=None, stderr=None, close_fds=True)
+            result = p.wait()
 
     def generate_command(self, paths):
         """Return a string containing the command to be launched."""
         raise NotImplementedError("must be impemented by sub-classes")
 
-    def run(self, executable, main_file, parameter_file=None, append_label=None):
+    def run(self, executable, main_file, arguments, append_label=None):
         """
         Run a computation in a shell, with the given executable, script and
-        parameter file. If `append_label` is provided, it is appended to the
+        arguments. If `append_label` is provided, it is appended to the
         command line. Return True if the computation finishes successfully,
         False otherwise.
         """
-        paths = check_files_exist(executable, main_file, parameter_file)
-        cmd = self.generate_command(paths)
+        cmd = self.generate_command(executable, main_file, arguments)
         if append_label:
-            cmd += append_label
+            cmd += " " + append_label
         print "Sumatra is running the following command:", cmd
         #p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, close_fds=True)
         p = subprocess.Popen(cmd, shell=True, stdout=None, stderr=None, close_fds=True)
@@ -99,7 +97,7 @@ class LaunchMode(object):
             return False
 
     def __eq__(self, other):
-        return self.__class__ == other.__class__ and self.get_state() == other.get_state()
+        return self.__class__ == other.__class__ and self.__getstate__() == other.__getstate__()
 
     def __ne__(self, other):
         return not self.__eq__(other)
@@ -132,9 +130,13 @@ class SerialLaunchMode(LaunchMode):
     def __str__(self):
         return "serial"
     
-    def generate_command(self, paths, append_label=None):
+    def __getstate__(self):
+        return {}
+    
+    def generate_command(self, executable, main_file, arguments):
         __doc__ = LaunchMode.__doc__
-        cmd = "%s "*len(paths) % tuple(paths)
+        check_files_exist(executable.path, *main_file.split())
+        cmd = "%s %s %s %s" % (executable.path, executable.options, main_file, arguments)
         return cmd
     
 
@@ -165,6 +167,7 @@ class DistributedLaunchMode(LaunchMode):
         else:
             mpi_cmd = MPI(path=None)
         self.mpirun = mpi_cmd.path #"/usr/local/bin/mpiexec"
+        # should warn if mpirun not found
         self.hosts = hosts
         self.n = n
         self.mpi_info = {}
@@ -173,8 +176,13 @@ class DistributedLaunchMode(LaunchMode):
     def __str__(self):
         return "distributed (n=%d, mpiexec=%s, hosts=%s)" % (self.n, self.mpirun, self.hosts)
     
-    def generate_command(self, paths):
+    def generate_command(self, executable, main_file, arguments):
         __doc__ = LaunchMode.__doc__
+        check_files_exist(self.mpirun, executable.path, *main_file.split())
+        if hasattr(executable, "mpi_options"):
+            mpi_options = executable.mpi_options
+        else:
+            mpi_options = ""
         #cmd = "%s -np %d -host %s %s %s %s" % (self.mpirun,
         #                                       self.n,
         #                                       ",".join(hosts),
@@ -185,7 +193,8 @@ class DistributedLaunchMode(LaunchMode):
             self.mpirun,
             self.n
         )
-        cmd += " %s"*len(paths) % tuple(paths)
+        cmd += " %s %s %s %s %s" % (executable.path, mpi_options,
+                                    executable.options, main_file, arguments)
         return cmd
     
     def get_platform_information(self):
@@ -211,11 +220,11 @@ class DistributedLaunchMode(LaunchMode):
                                        maxprocs=self.n)
             platform_information = []
             for rank in range(self.n):
-                platform_information.append(comm.recv(source=rank, tag=rank))
+                platform_information.append(PlatformInformation(**comm.recv(source=rank, tag=rank).values()[0]))
             comm.Disconnect()
         return platform_information
 
-    def get_state(self):
+    def __getstate__(self):
         """Return a dict containing the values needed to recreate this instance."""
-        return {'mpirun': self.mpirun, 'n': self.n, 'hosts': self.hosts}
+        return {'mpirun': self.mpirun, 'n': self.n, 'hosts': self.hosts, 'pfi_path': self.pfi_path}
     
