@@ -24,7 +24,6 @@ ConfigParserParameterSet - handles parameter files in traditional config file
 from __future__ import with_statement
 import os.path
 import shutil
-import re
 from ConfigParser import SafeConfigParser, MissingSectionHeaderError
 from cStringIO import StringIO
 from sumatra.external import NeuroTools
@@ -48,21 +47,32 @@ class SimpleParameterSet(object):
         self.values = {}
         self.types = {}
         self.comments = {}
-        if os.path.exists(initialiser):
-            with open(initialiser) as f:
-                content = f.readlines()
+        if isinstance(initialiser, dict):
+            for name, value in initialiser.items():
+                self.values[name] = value
+                self.types[name] = type(value)
         else:
-            content = initialiser.split("\n")
-        for line in content:
-            if "=" in line:
-                name, value = line.split("=")[:2]
-                name = name.strip()
-                if "#" in value:
-                    value, comment = value.split("#")[:2]
-                    self.comments[name] = comment
-                self.values[name] = eval(value)
-                self.types[name] = type(self.values[name])   
-    
+            if os.path.exists(initialiser):
+                with open(initialiser) as f:
+                    content = f.readlines()
+            else:
+                content = initialiser.split("\n")
+            for line in content:
+                line = line.strip()
+                if "=" in line:
+                    name, value = line.split("=")[:2]
+                    name = name.strip()
+                    if "#" in value:
+                        value, comment = value.split("#")[:2]
+                        self.comments[name] = comment
+                    self.values[name] = eval(value)
+                    self.types[name] = type(self.values[name])   
+                elif line:
+                    if line.strip()[0] == "#":
+                        pass
+                    else:
+                        raise SyntaxError("File is not a valid simple parameter file. This line caused the error: %s" % line)
+
     def __str__(self):
         return self.pretty()
     
@@ -214,10 +224,7 @@ class ConfigParserParameterSet(SafeConfigParser):
             _update(name, value)
 
 
-list_pattern = re.compile(r'^\s*\[.*\]\s*$')
-tuple_pattern = re.compile(r'^\s*\(.*\)\s*$')
-
-def build_parameters(filename, cmdline_parameters=[]):
+def build_parameters(filename):
     try:
         parameters = NTParameterSet("file://%s" % os.path.abspath(filename))
     except SyntaxError:
@@ -225,16 +232,4 @@ def build_parameters(filename, cmdline_parameters=[]):
             parameters = ConfigParserParameterSet(filename)
         except SyntaxError:
             parameters = SimpleParameterSet(filename)
-    for p in cmdline_parameters:
-        name, value = p.split("=")
-        if list_pattern.match(value) or tuple_pattern.match(value):
-            value = eval(value)
-        else:
-            for cast in int, float:
-                try:
-                    value = cast(value)
-                    break
-                except ValueError:
-                    pass
-        parameters.update({name: value})
-    return parameters
\ No newline at end of file
+    return parameters
