@@ -21,34 +21,38 @@ get_executable()      - Return an appropriate subclass of Executable, given
 register_executable() - Register new subclasses of Executable that can be
                         returned by get_executable().
 """
-
+from __future__ import with_statement
 import os.path
 import re
 import subprocess
 
 version_pattern = re.compile(r'\b(?P<version>\d[\.\d]*([a-z]*\d)*)\b')
 
-class VersionedProgram(object):
-    pass
-    
-        
-class Executable(VersionedProgram):
-    # store compilation/configuration options?
 
-    def __init__(self, path, version=None):
-        VersionedProgram.__init__(self)
+class Executable(object):
+    # store compilation/configuration options? yes, if we can determine them
+
+    def __init__(self, path, version=None, options=""):
         if not hasattr(self, 'name'):
             self.name = os.path.basename(path)
         if path and os.path.exists(path):
             self.path = path
         else:
-            self.path = self._find_executable(path or self.default_executable_name)    
+            try:
+                self.path = self._find_executable(path or self.default_executable_name)
+            except Warning, errmsg:
+                print errmsg
+                self.path = path
         if not hasattr(self, 'name'):
             self.name = os.path.basename(self.path)
         self.version = version or self._get_version()
+        self.options = options
 
     def __str__(self):
-        return "%s (version: %s) at %s" % (self.name, self.version, self.path)
+        s = "%s (version: %s) at %s" % (self.name, self.version, self.path)
+        if self.options:
+            s += "options: %s" % self.options
+        return s
 
     def _find_executable(self, executable_name):
         found = []
@@ -56,7 +60,7 @@ class Executable(VersionedProgram):
             if os.path.exists(os.path.join(path, executable_name)):
                 found += [path] 
         if not found:
-            raise Exception('%s could not be found. Please supply the path to the %s executable.' % (self.name, executable_name))
+            raise Warning('%s could not be found. Please supply the path to the %s executable.' % (self.name, executable_name))
         else:
             executable = os.path.join(found[0], executable_name) 
             if len(found) == 1:
@@ -76,26 +80,41 @@ class Executable(VersionedProgram):
         return version
 
     def __eq__(self, other):
-        return type(self) == type(other) and self.path == other.path and self.name == other.name and self.version == other.version
+        return type(self) == type(other) and self.path == other.path and self.name == other.name and self.version == other.version and self.options == other.options
     
     def __ne__(self, other):
         return not self.__eq__(other)
     
+    def __getstate__(self):
+        return {'path': self.path, 'version': self.version, 'options': self.options}
+    
+    @staticmethod
+    def write_parameters(parameters, filename):
+        parameters.save(filename)
+
 
 class NEURONSimulator(Executable):
     
     name = "NEURON"
     default_executable_name = "nrniv"
-    
+    mpi_options = "-mpi"
+    pre_run = "nrnivmodl"
+
+    @staticmethod
+    def write_parameters(parameters, filename):
+        with open(filename, 'w') as fp:
+            for name, value in parameters.as_dict().items():
+                if isinstance(value, basestring):
+                    fp.write('strdef %s\n' % name)
+                    fp.write('%s = "%s"\n' % (name, value))
+                else:
+                    fp.write('%s = %g\n' % (name, value))
+
 
 class PythonExecutable(Executable):
     
     name = "Python"
-    default_executable_name = "python"
-    
-    @staticmethod
-    def write_parameters(parameters, filename):
-        parameters.save(filename)
+    default_executable_name = "python" 
 
 
 class NESTSimulator(Executable):
@@ -103,8 +122,28 @@ class NESTSimulator(Executable):
     name = "NEST"
     default_executable_name = 'nest'
     
-    
 
+class GENESISSimulator(Executable):
+    
+    name = "GENESIS"
+    default_executable_name = "genesis"
+
+    def _get_version(self):
+        print "Writing genesis version script"
+        with open("genesis_version.g", "w") as fd:
+            fd.write("""openfile genesis_version.out w
+                        writefile genesis_version.out {version}
+                        closefile genesis_version.out
+                        quit
+                    """)
+        p = subprocess.Popen("%s genesis_version.g" % self.path, shell=True) #stdout=subprocess.PIPE, stderr=subprocess.STDOUT, stdin=subprocess.PIPE, shell=True)
+        returncode = p.wait()
+        print returncode
+        with open("genesis_version.out") as fd:
+            version = fd.read()
+        os.remove("genesis_version.g")
+        os.remove("genesis_version.out")
+        return version.strip()
     
 registered_program_names = {}
 registered_executables = {}
@@ -121,6 +160,7 @@ def register_executable(cls, name, executable, extensions):
 register_executable(NEURONSimulator, 'NEURON', 'nrniv', ('.hoc', '.oc'))
 register_executable(PythonExecutable, 'Python', 'python', ('.py',))
 register_executable(NESTSimulator, 'NEST', 'nest', ('.sli',))
+register_executable(GENESISSimulator, 'GENESIS', 'genesis', ('.g',))
     
 def get_executable(path=None, script_file=None):
     """
