@@ -20,18 +20,28 @@ load_project() - read project information from the working directory and return
 """
 
 import os
+import sys
 import cPickle as pickle
 from copy import deepcopy
-from datastore import FileSystemDataStore
-from records import Record
-from formatting import get_formatter, get_diff_formatter
-from recordstore import DefaultRecordStore
-from versioncontrol import UncommittedModificationsError, get_working_copy
+from sumatra.records import Record
+from sumatra import programs, datastore
+from sumatra.formatting import get_formatter, get_diff_formatter
+from sumatra.recordstore import DefaultRecordStore
+from sumatra.versioncontrol import UncommittedModificationsError, get_working_copy
+import mimetypes
+try:
+    import json
+except ImportError:
+    import simplejson as json
+
+DEFAULT_PROJECT_FILE = "project"
 
 def _remove_left_margin(s): # replace this by textwrap.dedent?
     lines = s.strip().split('\n')
     return "\n".join(line.strip() for line in lines)
 
+def _get_project_file(path):
+    return os.path.join(path, ".smt", DEFAULT_PROJECT_FILE)
 
 class Project(object):
 
@@ -39,9 +49,10 @@ class Project(object):
                  default_main_file=None, default_launch_mode=None,
                  data_store='default', record_store='default',
                  on_changed='error', description='', data_label=None):
+        self.path = os.getcwd()
         if not os.path.exists(".smt"):
             os.mkdir(".smt")
-        if os.path.exists(".smt/project"):
+        if os.path.exists(_get_project_file(self.path)):
             raise Exception("Sumatra project already exists in this directory.")
         self.name = name
         self.default_executable = default_executable
@@ -49,7 +60,7 @@ class Project(object):
         self.default_main_file = default_main_file
         self.default_launch_mode = default_launch_mode
         if data_store == 'default':
-            data_store = FileSystemDataStore(None)
+            data_store = datastore.FileSystemDataStore(None)
         self.data_store = data_store # a data store object
         if record_store == 'default':
             record_store = DefaultRecordStore(".smt/records")
@@ -57,6 +68,7 @@ class Project(object):
         self.on_changed = on_changed
         self.description = description
         self.data_label = data_label
+        self._most_recent = None
         self.save()
         print "Sumatra project successfully set up"
     
@@ -69,8 +81,20 @@ class Project(object):
     
     def save(self):
         """Save state to some form of persistent storage. (file, database)."""
-        f = open('.smt/project', 'w') # should check if file exists?
-        pickle.dump(self, f)
+        state = {}
+        for name in ('name', 'default_executable', 'default_repository',
+                     'default_launch_mode', 'data_store', 'record_store',
+                     'default_main_file', 'on_changed', 'description',
+                     'data_label', '_most_recent'):
+            attr = getattr(self, name)
+            if hasattr(attr, "__getstate__"):
+                state[name] = {'type': attr.__class__.__module__ + "." + attr.__class__.__name__}
+                for key, value in attr.__getstate__().items():
+                    state[name][key] = value
+            else:
+                state[name] = attr
+        f = open(_get_project_file(self.path), 'w') # should check if file exists?
+        json.dump(state, f, indent=2)
         f.close()
     
     def info(self):
@@ -90,7 +114,8 @@ class Project(object):
         """
         return _remove_left_margin(template % self.__dict__)
     
-    def new_record(self, parameters, executable='default', repository='default',
+    def new_record(self, parameters={}, input_data=[], script_args="",
+                   executable='default', repository='default',
                    main_file='default', version='latest', launch_mode='default',
                    label=None, reason=None):
         if executable == 'default':
@@ -101,20 +126,22 @@ class Project(object):
             main_file = self.default_main_file
         if launch_mode == 'default':
             launch_mode = deepcopy(self.default_launch_mode)
-        version, diff = self.update_code(repository.working_copy, version)
-        record = Record(executable, repository, main_file, version, parameters,
-                        launch_mode, self.data_store, label=label, reason=reason,
-                        diff=diff, on_changed=self.on_changed)
-        record.register()
+        working_copy = repository.get_working_copy()
+        version, diff = self.update_code(working_copy, version)
+        record = Record(executable, repository, main_file, version, launch_mode,
+                        self.data_store, parameters, input_data, script_args, 
+                        label=label, reason=reason, diff=diff,
+                        on_changed=self.on_changed)
+        record.register(working_copy)
         return record
     
-    def launch(self, parameters, executable='default', repository='default',
-               main_file='default', version='latest', launch_mode='default',
-               label=None, reason=None):
+    def launch(self, parameters={}, input_data=[], script_args="",
+               executable='default', repository='default', main_file='default',
+               version='latest', launch_mode='default', label=None, reason=None):
         """Launch a new simulation or analysis."""
-        record = self.new_record(parameters, executable, repository,
-                                 main_file, version, launch_mode, label,
-                                 reason)
+        record = self.new_record(parameters, input_data, script_args,
+                                 executable, repository, main_file, version,
+                                 launch_mode, label, reason)
         record.run(with_label=self.data_label)
         self.add_record(record)
         self.save()
@@ -155,6 +182,7 @@ class Project(object):
         if delete_data:
             self.get_record(label).delete_data()
         self.record_store.delete(self.name, label)
+        self._most_recent = self.record_store.most_recent(self.name)
     
     def delete_by_tag(self, tag, delete_data=False):
         """Delete all records with a given tag. Return the number of records deleted."""
@@ -162,6 +190,7 @@ class Project(object):
             for record in self.record_store.list(self.name, tag):
                 record.delete_data()
         n = self.record_store.delete_by_tag(self.name, tag)
+        self._most_recent = self.record_store.most_recent(self.name)
         return n
     
     def format_records(self, format='text', mode='short', tags=None):
@@ -200,13 +229,52 @@ class Project(object):
         formatter = get_diff_formatter()(diff)
         return formatter.format(mode)
 
-    
-def load_project():
-    if os.path.exists(".smt"):
-        f = open(".smt/project", 'r')
-        prj = pickle.load(f)
-        f.close()
-        return prj
+def _load_project_from_json(path):
+    f = open(_get_project_file(path), 'r')
+    data = json.load(f)
+    f.close()
+    prj = Project.__new__(Project)
+    prj.path = path
+    for key, value in data.items():
+        if isinstance(value, dict) and "type" in value:
+            parts = str(value["type"]).split(".") # make sure not unicode, see http://stackoverflow.com/questions/1971356/haystack-whoosh-index-generation-error/2683624#2683624
+            module_name = ".".join(parts[:-1])
+            class_name = parts[-1]
+            _temp = __import__(module_name, globals(), locals(), [class_name], -1) # from <module_name> import <class_name>
+            cls = getattr(_temp, class_name)
+            args = {}
+            for k,v in value.items():
+                if k != 'type':
+                    args[str(k)] = v # need to use str() as json module uses all unicode
+            setattr(prj, key, cls(**args))
+        else:
+            setattr(prj, key, value)
+    return prj
+
+def _load_project_from_pickle(path):
+    # earlier versions of Sumatra saved Projects using pickle
+    f = open(_get_project_file(path), 'r')
+    prj = pickle.load(f)
+    f.close()
+    return prj
+
+def load_project(path=None):
+    """
+    Read project from directory passed as the argument and return Project
+    object. If no argument is given, the project is read from the current
+    directory.
+    """
+    if not path:
+        p = os.getcwd()
     else:
-        raise Exception("No Sumatra project exists in the current directory")
-        
\ No newline at end of file
+        p = path
+    while not os.path.isdir(os.path.join(p, ".smt")):
+        oldp, p = p, os.path.dirname(p)
+        if p == oldp:
+            raise Exception("No Sumatra project exists in the current directory or above it.")
+    mimetypes.init([os.path.join(p, ".smt", "mime.types")])
+    #try:
+    prj = _load_project_from_json(p)
+    #except Exception:
+    #    prj = _load_project_from_pickle(p)
+    return prj
