@@ -24,10 +24,13 @@ def assert_equal(a, b, msg=''):
     assert a == b, "%s: %s %s != %s %s" % (msg, a, type(a), b, type(b))
 
 class Record(object):
+    """
     
-    def __init__(self, executable, repository, main_file, version, parameters,
-                 launch_mode, datastore, label=None, reason=None, diff='',
-                 user='', on_changed='error'):
+    """
+    
+    def __init__(self, executable, repository, main_file, version, launch_mode,
+                 datastore, parameters={}, input_data=[], script_arguments="",
+                 label=None, reason=None, diff='', user='', on_changed='error'):
         self.timestamp = datetime.now() # might need to allow for this to be set as argument to allow for distributed/batch simulations on machines with out-of-sync clocks
         self.label = label or self.timestamp.strftime("%Y%m%d-%H%M%S")
         assert len(self.label) > 0
@@ -37,7 +40,9 @@ class Record(object):
         self.repository = repository # a Repository object
         self.main_file = main_file
         self.version = version
-        self.parameters = parameters # a ParameterSet object
+        self.parameters = parameters
+        self.input_data = input_data # a list containing strings (in future - DataFile objects)
+        self.script_arguments = script_arguments
         self.launch_mode = launch_mode # a LaunchMode object - basically, run serially or with MPI. If MPI, what configuration
         self.datastore = datastore.copy()
         self.outcome = None
@@ -47,14 +52,21 @@ class Record(object):
         self.user = user
         self.on_changed = on_changed  
     
-    def register(self):
+    def register(self, working_copy):
         """Record information about the environment."""
         # Check the code hasn't changed and the version is correct
         if len(self.diff) == 0:
-            assert not self.repository.working_copy.has_changed()
-        assert_equal(self.repository.working_copy.current_version(), self.version, "version")
+            assert not working_copy.has_changed()
+        assert_equal(working_copy.current_version(), self.version, "version")
         # Record dependencies
-        self.dependencies = dependency_finder.find_dependencies(self.main_file, self.executable, self.on_changed)
+        if len(self.main_file.split()) == 1: # this assumes filenames cannot contain spaces
+            self.dependencies = dependency_finder.find_dependencies(self.main_file, self.executable)
+        else: # if self.main_file contains multiple file names
+            # this seems a bit hacky. Should perhaps store a list self.main_files, _and_ check that all files exist.
+            self.dependencies = []
+            for main_file in self.main_file.split():
+                self.dependencies.extend(dependency_finder.find_dependencies(main_file, self.executable))
+        # if self.on_changed is 'error', should check that all the dependencies have empty diffs and raise an UncommittedChangesError otherwise
         # Record platform information
         self.platforms = self.launch_mode.get_platform_information()
     
@@ -81,24 +93,37 @@ class Record(object):
         # run pre-simulation/analysis tasks, e.g. nrnivmodl
         self.launch_mode.pre_run(self.executable)
         # Write the executable-specific parameter file
-        parameter_file = "%s.param" % self.label.replace("/", "_")
-        self.executable.write_parameters(self.parameters, parameter_file)
+        script_arguments = self.script_arguments
+        if self.parameters:
+            parameter_file = "%s.param" % self.label.replace("/", "_")
+            self.executable.write_parameters(self.parameters, parameter_file)
+            script_arguments = script_arguments.replace("<parameters>", parameter_file)
         # Run simulation/analysis
         start_time = time.time()
-        result = self.launch_mode.run(self.executable, self.main_file, parameter_file, data_label)
+        result = self.launch_mode.run(self.executable, self.main_file,
+                                      script_arguments, data_label)
         self.duration = time.time() - start_time
         # Run post-processing scripts
         pass # skip this if there is an error
         # Search for newly-created datafiles
+        if self.parameters and os.path.exists(parameter_file):
+            os.remove(parameter_file)
         self.data_key = self.datastore.find_new_files(self.timestamp)
         print "Data key is", self.data_key
-        if os.path.exists(parameter_file):
-            os.remove(parameter_file)
+    
+    def __repr__(self):
+        return "Record #%s" % self.label
     
     def describe(self, format='text', mode='long'):
         formatter = get_formatter(format)([self])
         return formatter.format(mode)
     
+    def __ne__(self, other):
+        return bool(self.difference(other))
+    
+    def __eq__(self, other):
+        return not self.__ne__(other)
+    
     def difference(self, other_record, ignore_mimetypes=[], ignore_filenames=[]):
         """
         Determine the difference between this computational experiment and
@@ -114,7 +139,7 @@ class Record(object):
         """
         self.datastore.delete(self.data_key)
         self.data_key = self.datastore.empty_key
-         
+
 
 class RecordDifference(object):
     """Represents the difference between two Record objects."""
@@ -125,7 +150,6 @@ class RecordDifference(object):
     def __init__(self, recordA, recordB,
                  ignore_mimetypes=[],
                  ignore_filenames=[]):
-        assert recordA.label != recordB.label
         self.recordA = recordA
         self.recordB = recordB
         assert not isinstance(ignore_mimetypes, basestring) # catch a 
@@ -137,8 +161,11 @@ class RecordDifference(object):
         self.main_file_differs = recordA.main_file != recordB.main_file
         self.version_differs = recordA.version != recordB.version
         for rec in recordA, recordB:
-            rec.parameters.pop("sumatra_label", 1)
+            if rec.parameters:
+                rec.parameters.pop("sumatra_label", 1)
         self.parameters_differ = recordA.parameters != recordB.parameters
+        self.input_data_differ = recordA.input_data != recordB.input_data
+        self.script_arguments_differ = recordA.script_arguments != recordB.script_arguments
         self.launch_mode_differs = recordA.launch_mode != recordB.launch_mode
         #self.platforms
         #self.datastore = datastore
@@ -155,8 +182,25 @@ class RecordDifference(object):
         practice, but then the output data will be different).
         """
         return reduce(or_, (self.executable_differs, self.code_differs,
+                            self.input_data_differ, self.script_arguments_differ,
                             self.parameters_differ, self.data_differs))
     
+    def __repr__(self):
+        s = "RecordDifference(%s, %s):" % (self.recordA.label, self.recordB.label)
+        if self.executable_differs:
+            s += 'X'
+        if self.code_differs:
+            s += 'C'
+        if self.parameters_differ:
+            s += 'P'
+        if self.data_differs:
+            s += 'D'
+        if self.input_data_differ:
+            s += 'I'
+        if self.script_arguments_differ:
+            s += 'S'
+        return s
+    
     @property
     def code_differs(self):
         return reduce(or_, (self.repository_differs, self.main_file_differs,
@@ -165,12 +209,7 @@ class RecordDifference(object):
     
     @property
     def dependencies_differ(self):
-        if len(self.recordA.dependencies) != len(self.recordB.dependencies):
-            return True
-        for depA,depB in zip(self.recordA.dependencies, self.recordB.dependencies):
-            if depA != depB:
-                return True
-        return False
+        return set(self.recordA.dependencies) != set(self.recordB.dependencies)
     
     @property
     def dependency_differences(self):
