@@ -11,7 +11,7 @@ from tagging.models import Tag
 
 class SumatraObjectsManager(models.Manager):
     
-    def get_or_create_from_sumatra_object(self, obj):
+    def get_or_create_from_sumatra_object(self, obj, using='default'):
         # automatically retrieving the field names is nice, but leads
         # to all the special cases below when we have subclasses that we
         # want to store in a single table in the database.
@@ -26,14 +26,14 @@ class SumatraObjectsManager(models.Manager):
                 attributes[name] = getattr(obj, name)
             except AttributeError:
                 if name == 'parameters':
-                    attributes[name] = str(obj.get_state())
+                    attributes[name] = str(obj.__getstate__())
                 elif name == 'type':
                     attributes[name] = obj.__class__.__name__
                 elif name == 'content':
                     attributes[name] = str(obj) # ParameterSet
                 else:
                     raise
-        return self.get_or_create(**attributes)            
+        return self.using(using).get_or_create(**attributes)            
         
 
 class BaseModel(models.Model):
@@ -54,15 +54,19 @@ class Project(BaseModel):
     name = models.CharField(max_length=200)
     description = models.TextField(blank=True)
 
+    def get_name(self):
+        return self.name or self.id
+
 
 class Executable(BaseModel):
     path = models.CharField(max_length=200)
     name = models.CharField(max_length=50)
     version = models.CharField(max_length=20)
+    options = models.CharField(max_length=50)
 
     def to_sumatra(self):
         cls = programs.registered_program_names.get(self.name, programs.Executable)
-        ex = cls(self.path, self.version)
+        ex = cls(self.path, self.version, self.options)
         ex.name = self.name
         return ex
 
@@ -106,6 +110,10 @@ class ParameterSet(BaseModel):
     def to_sumatra(self):
         if hasattr(parameters, self.type):
             ps = getattr(parameters, self.type)(self.content)
+        elif self.content == u'None':
+            ps = None
+        elif self.content == u'{}':
+            ps = {}
         else:
             ps = self.content
         return ps
@@ -171,6 +179,7 @@ class Record(BaseModel):
     main_file = models.CharField(max_length=100)
     version = models.CharField(max_length=50)
     parameters = models.ForeignKey(ParameterSet)
+    input_data = models.TextField(blank=True)
     launch_mode = models.ForeignKey(LaunchMode)
     datastore = models.ForeignKey(Datastore)
     outcome = models.TextField(blank=True)
@@ -182,6 +191,7 @@ class Record(BaseModel):
     diff = models.TextField(blank=True)
     user = models.CharField(max_length=30)
     project = models.ForeignKey(Project, null=True)
+    script_arguments = models.TextField(blank=True)
 
     def to_sumatra(self):
         record = records.Record(
@@ -189,9 +199,11 @@ class Record(BaseModel):
             self.repository.to_sumatra(),
             self.main_file,
             self.version,
-            self.parameters.to_sumatra(),
             self.launch_mode.to_sumatra(),
             self.datastore.to_sumatra(),
+            self.parameters.to_sumatra(),
+            eval(self.input_data),
+            self.script_arguments,
             self.label,
             self.reason,
             self.diff,
