@@ -19,9 +19,23 @@ get_repository()        - return a GitRepository object for a given URL.
 import git
 import os
 import shutil
+try:
+    from git.errors import InvalidGitRepositoryError
+except:
+    from git.exc import InvalidGitRepositoryError
+
 
 from base import Repository, WorkingCopy, VersionControlError
 
+def findrepo(path):
+    try:
+        repo = git.Repo(path)
+    except InvalidGitRepositoryError:
+        return
+    else:
+        return os.path.dirname(repo.git_dir)
+        
+
 def check_version():
     if git.__version__.split(".")[1] < 2:
         raise VersionControlError("Your Git Python binding is too old. You require at least version 0.2.0-beta1.")
@@ -30,15 +44,19 @@ def may_have_working_copy(path=None):
     """Test whether there is a Git working copy at the given path."""
     check_version()
     path = path or os.getcwd()
-    if git.repo.is_git_dir(os.path.join(path, ".git")):
-        return os.path.exists(os.path.join(path, ".git"))
+    if findrepo(path):
+        return True
     else:
         return False
 
 def get_working_copy(path=None):
     """Return a GitWorkingCopy instance for the given path, or the current
     directory if the path is not given."""
-    return GitWorkingCopy(path)
+    repo_dir = findrepo(path or os.getcwd())
+    if repo_dir:
+        return GitWorkingCopy(repo_dir)
+    else:
+        raise VersionControlError("No Git working copy found at %s" % path)
 
 def get_repository(url):
     """Return a GitRepository instance for the given url."""
@@ -50,12 +68,11 @@ class GitWorkingCopy(WorkingCopy):
     An object which allows various operations on a Git working copy.
     """
 
-    def __init__(self, path=None, repository=None):
+    def __init__(self, path=None):
         check_version()
         WorkingCopy.__init__(self)
         self.path = path or os.getcwd()
-        self.repository = repository or GitRepository(self.path)
-        self.repository.working_copy = self
+        self.repository = GitRepository(self.path)
 
     def current_version(self):
         head = self.repository._repository.head
@@ -93,13 +110,13 @@ def move_contents(src, dst):
             shutil.copy2(src_path, dst)
     shutil.rmtree(src)
 
+
 class GitRepository(Repository):
     
     def __init__(self, url):
         check_version()
         Repository.__init__(self, url)
-        self.working_copy = None
-        self.checkout(url)
+        self.checkout(url) # should remove this line - other repos don't call checkout on init
             
     def checkout(self, path="."):
         """Clone a repository."""
@@ -113,13 +130,6 @@ class GitRepository(Repository):
             g.clone(self.url, tmpdir)
             move_contents(tmpdir, path)
         self._repository = git.Repo(path)
-        self.working_copy = GitWorkingCopy(path, repository=self)
-    
-    def __getstate__(self):
-        """For pickling"""
-        return {'url': self.url, 'wc': self.working_copy}
-    
-    def __setstate__(self, state):
-        """For unpickling"""
-        self.__init__(state['url'])
-        self.working_copy = state['wc']
+
+    def get_working_copy(self, path=None):
+        return get_working_copy(path)
