@@ -3,15 +3,30 @@ Unit tests for the sumatra.versioncontrol package
 """
 
 from __future__ import with_statement
-import unittest
+import unittest2 as unittest
 import os
 import pickle
 import tempfile
 import shutil
+
 from sumatra.versioncontrol._mercurial import MercurialRepository, MercurialWorkingCopy, may_have_working_copy
-from sumatra.versioncontrol._subversion import SubversionRepository, SubversionWorkingCopy
-from sumatra.versioncontrol._git import GitRepository, GitWorkingCopy
-from sumatra.versioncontrol import get_repository
+try:
+    from sumatra.versioncontrol._subversion import SubversionRepository, SubversionWorkingCopy
+    have_pysvn = True
+except ImportError:
+    have_pysvn = False
+try:
+    from sumatra.versioncontrol._git import GitRepository, GitWorkingCopy
+    have_git = True
+except ImportError:
+    have_git = False
+try:
+    from sumatra.versioncontrol._bazaar import BazaarRepository, BazaarWorkingCopy
+    have_bzr = True
+except ImportError:
+    have_bzr = False
+from sumatra.versioncontrol import get_repository, get_working_copy
+
 
 class BaseTestWorkingCopy(object):
     
@@ -29,9 +44,6 @@ class BaseTestWorkingCopy(object):
                 name, value = line.split("=")
                 P[name.strip()] = eval(value)
         return P
-    
-    def test__init(self):
-        self.assertEqual(self.wc.repository.working_copy, self.wc)
         
     def test__has_changed(self):
         self.assertEqual(self.wc.has_changed(), False)
@@ -84,7 +96,7 @@ class TestMercurialWorkingCopy(unittest.TestCase, BaseTestWorkingCopy):
         self.assertEqual(self.wc.status(), {'modified': ['romans.param'], 'removed': [],
                                             'deleted': [], 'unknown': []})
 
-
+#@unittest.skipUnless(have_git, "Could not import git") # 
 class TestGitWorkingCopy(unittest.TestCase, BaseTestWorkingCopy):
     
     def setUp(self):
@@ -100,8 +112,10 @@ class TestGitWorkingCopy(unittest.TestCase, BaseTestWorkingCopy):
     def tearDown(self):
         os.unlink("%s/.git" % self.repository_path)
         shutil.rmtree(self.tmpdir)
+TestGitWorkingCopy = unittest.skipUnless(have_git, "Could not import git")(TestGitWorkingCopy) # not using as class decorator for the sake of Python 2.5
 
 
+#@unittest.skipUnless(have_pysvn, "Could not import pysvn")
 class TestSubversionWorkingCopy(unittest.TestCase, BaseTestWorkingCopy):
     
     def setUp(self):
@@ -120,140 +134,194 @@ class TestSubversionWorkingCopy(unittest.TestCase, BaseTestWorkingCopy):
         self.assertEqual(self.wc.status()['modified'], [])
         self.change_file()
         self.assertEqual(os.path.basename(self.wc.status()['modified'][0]), 'romans.param')
-        
+TestSubversionWorkingCopy = unittest.skipUnless(have_pysvn, "Could not import pysvn")(TestSubversionWorkingCopy)
 
-class TestMercurialRepository(unittest.TestCase):
+
+#@unittest.skipUnless(have_bzr, "Could not import bzrlib")
+class TestBazaarWorkingCopy(unittest.TestCase, BaseTestWorkingCopy):
+    
+    def setUp(self):
+        self.repository_path = os.path.abspath("../example_repositories/bazaar")
+        os.symlink("%s/bzr" % self.repository_path, "%s/.bzr" % self.repository_path)
+        self.repos = BazaarRepository(self.repository_path)
+        self.tmpdir = tempfile.mkdtemp()
+        self.repos.checkout(self.tmpdir)
+        self.wc = BazaarWorkingCopy(self.tmpdir)
+        self.latest_version = "4"
+        self.previous_version = "1"
+        
+    def tearDown(self):
+        os.unlink("%s/.bzr" % self.repository_path)
+        shutil.rmtree(self.tmpdir)
+TestBazaarWorkingCopy = unittest.skipUnless(have_bzr, "Could not import bzrlib")(TestBazaarWorkingCopy)
+
+
+class BaseTestRepository(object):
+    
+    def test__pickling(self):
+        r = self._create_repository()
+        dump = pickle.dumps(r)
+        r2 = pickle.loads(dump)
+        self.assertEqual(r.url, r2.url)
+    
+    def test__init(self):
+        r = self._create_repository()
+        self.assertEqual(r.url, self.repository_path)
+
+    def test__checkout_of_remote_repos(self):
+        tmpdir = tempfile.mkdtemp()
+        r = self._create_repository()
+        r.checkout(path=tmpdir)
+        repos_files = os.listdir(tmpdir)
+        repos_files.remove(self.special_dir)
+        project_files = os.listdir("../example_projects/python")
+        if "main.pyc" in project_files:
+            project_files.remove("main.pyc")
+        self.assertEqual(set(repos_files), set(project_files))
+        shutil.rmtree(tmpdir)
+
+    def test__str(self):
+        r = self._create_repository()
+        str(r)
+
+    def test__eq(self):
+        r1 = self._create_repository()
+        r2 = self._create_repository()
+        self.assertEqual(r1, r2)
+
+
+class TestMercurialRepository(unittest.TestCase, BaseTestRepository):
     
     def setUp(self):
         self.repository_path = os.path.abspath("../example_repositories/mercurial")
         os.symlink("%s/hg" % self.repository_path, "%s/.hg" % self.repository_path)
+        self.special_dir = ".hg"
         
     def tearDown(self):
         os.unlink("%s/.hg" % self.repository_path)
+    
+    def _create_repository(self):
+        return MercurialRepository("file://%s" % self.repository_path)
         
     def test__init(self):
-        r = MercurialRepository("file://%s" % self.repository_path)
+        r = self._create_repository()
         self.assertEqual(r._repository.url(), "file:%s" % self.repository_path)
 
     def test__init__with_nonexistent_repos__should_raise_Exception(self):
         self.assertRaises(Exception, MercurialRepository, "file:///tmp/")
-
-    def test__pickling(self):
-        r = MercurialRepository("file://%s" % self.repository_path)
-        dump = pickle.dumps(r)
-        r2 = pickle.loads(dump)
-        self.assertEqual(r.url, r2.url)
-        self.assertEqual(r.working_copy, r2.working_copy)
-        
-    def test__checkout_of_remote_repos(self):
+    
+    def test__can_create_project_in_subdir(self):
+        #Test if a Sumatra project can be created in one of the subdirectories of a repository
         tmpdir = tempfile.mkdtemp()
-        r = MercurialRepository("file://%s" % self.repository_path)
+        r = self._create_repository()
         r.checkout(path=tmpdir)
-        repos_files = os.listdir(tmpdir)
-        repos_files.remove(".hg")
-        project_files = os.listdir("../example_projects/python")
-        if "main.pyc" in project_files:
-            project_files.remove("main.pyc")
-        self.assertEqual(set(repos_files), set(project_files))
+        # get a working copy from the subdirectory
+        wc = get_working_copy(os.path.join(tmpdir, 'subpackage')).repository.url
+        # is the working copy path same as the repo path?
+        self.assertEqual(wc, tmpdir)
         shutil.rmtree(tmpdir)
-
-    def test__str(self):
-        r = MercurialRepository("file://%s" % self.repository_path)
-        str(r)
         
-    def test__eq(self):
-        r1 = MercurialRepository("file://%s" % self.repository_path)
-        r2 = MercurialRepository("file://%s" % self.repository_path)
-        self.assertEqual(r1, r2)
 
-
-class TestGitRepository(unittest.TestCase):
+#@unittest.skipUnless(have_git, "Could not import git")
+class TestGitRepository(unittest.TestCase, BaseTestRepository):
     
     def setUp(self):
         self.repository_path = os.path.abspath("../example_repositories/git")
-        os.symlink("%s/git" % self.repository_path, "%s/.git" % self.repository_path)
+        try:
+            os.symlink("%s/git" % self.repository_path, "%s/.git" % self.repository_path)
+        except OSError:
+            pass
+        self.special_dir = ".git"
         
     def tearDown(self):
         os.unlink("%s/.git" % self.repository_path)
-        
-    def test__init(self):
-        r = GitRepository(self.repository_path)
-        self.assertEqual(r.url, self.repository_path)
+    
+    def _create_repository(self):
+        return GitRepository(self.repository_path)
 
     def test__init__with_nonexistent_repos__should_raise_Exception(self):
         self.assertRaises(Exception, GitRepository, "/tmp/")
 
-    def test__pickling(self):
-        r = GitRepository(self.repository_path)
-        dump = pickle.dumps(r)
-        r2 = pickle.loads(dump)
-        self.assertEqual(r.url, r2.url)
-        self.assertEqual(r.working_copy, r2.working_copy)
-        
-    def test__checkout_of_remote_repos(self):
+    def test__can_create_project_in_subdir(self):
+        #Test if a Sumatra project can be created in one of the subdirectories of a repository
         tmpdir = tempfile.mkdtemp()
-        r = GitRepository(self.repository_path)
+        r = self._create_repository()
         r.checkout(path=tmpdir)
-        repos_files = os.listdir(tmpdir)
-        repos_files.remove(".git")
-        project_files = os.listdir("../example_projects/python")
-        if "main.pyc" in project_files:
-            project_files.remove("main.pyc")
-        self.assertEqual(set(repos_files), set(project_files))
+        # get a working copy from the subdirectory
+        wc = get_working_copy(os.path.join(tmpdir, 'subpackage')).repository.url
+        # is the working copy path same as the repo path?
+        self.assertEqual(wc, tmpdir)
         shutil.rmtree(tmpdir)
+TestGitRepository = unittest.skipUnless(have_git, "Could not import git")(TestGitRepository)
 
-    def test__str(self):
-        r = GitRepository(self.repository_path)
-        str(r)
-        
-    def test__eq(self):
-        r1 = GitRepository(self.repository_path)
-        r2 = GitRepository(self.repository_path)
-        self.assertEqual(r1, r2)
-        
 
-class TestSubversionRepository(unittest.TestCase):
+#@unittest.skipUnless(have_pysvn, "Could not import pysvn")
+class TestSubversionRepository(unittest.TestCase, BaseTestRepository):
     
     def setUp(self):
         self.repository_path = os.path.abspath("../example_repositories/subversion")
+        self.special_dir = ".svn"
+        
+    def _create_repository(self):
+        return SubversionRepository("file://%s" % self.repository_path)
     
     def test__init(self):
-        r = SubversionRepository("file://%s" % self.repository_path)
+        r = self._create_repository()
         assert hasattr(r._client, "checkout")
         
     def test__init__with_nonexistent_repos__should_raise_Exception(self):
         self.assertRaises(Exception, SubversionRepository, "file:///tmp/")
+   
+    def test__checkout__with_nonexistent_repos__should_raise_Exception(self):
+        r = self._create_repository()
+        self.assertRaises(Exception, r.checkout, path="file:///tmp/")
+        shutil.rmtree("file:") # this is not quite what's supposed to happen
+TestSubversionRepository = unittest.skipUnless(have_pysvn, "Could not import pysvn")(TestSubversionRepository)
 
-    def test__pickling(self):
-        r = SubversionRepository("file://%s" % self.repository_path)
-        dump = pickle.dumps(r)
-        r2 = pickle.loads(dump)
-        self.assertEqual(r.url, r2.url)
-        self.assertEqual(r.working_copy, r2.working_copy)
+
+#@unittest.skipUnless(have_bzr, "Could not import bzrlib")
+class TestBazaarRepository(unittest.TestCase, BaseTestRepository):
+    
+    def setUp(self):
+        self.repository_path = os.path.abspath("../example_repositories/bazaar")
+        try:
+            os.symlink("%s/bzr" % self.repository_path, "%s/.bzr" % self.repository_path)
+        except OSError:
+            pass
+        self.special_dir = ".bzr"
         
+    def tearDown(self):
+        os.unlink("%s/.bzr" % self.repository_path)
+    
+    def _create_repository(self):
+        return BazaarRepository(self.repository_path)
+
+    def test__init__with_nonexistent_repos__should_raise_Exception(self):
+        self.assertRaises(Exception, BazaarRepository, "/tmp/")
+
     def test__checkout_of_remote_repos(self):
         tmpdir = tempfile.mkdtemp()
-        r = SubversionRepository("file://%s" % self.repository_path)
+        r = self._create_repository()
         r.checkout(path=tmpdir)
         repos_files = os.listdir(tmpdir)
-        repos_files.remove(".svn")
+        repos_files.remove(self.special_dir)
         project_files = os.listdir("../example_projects/python")
+        project_files.remove('EGG-INFO')
         if "main.pyc" in project_files:
             project_files.remove("main.pyc")
         self.assertEqual(set(repos_files), set(project_files))
         shutil.rmtree(tmpdir)
-        
-    def test__checkout__with_nonexistent_repos__should_raise_Exception(self):
-        r = SubversionRepository("file://%s" % self.repository_path)
-        self.assertRaises(Exception, r.checkout, path="file:///tmp/")
-        shutil.rmtree("file:") # this is not quite what's supposed to happen
+TestBazaarRepository = unittest.skipUnless(have_bzr, "Could not import bzrlib")(TestBazaarRepository)
 
 
 class TestMercurialModuleFunctions(unittest.TestCase):
     
     def setUp(self):
         self.repository_path = os.path.abspath("../example_repositories/mercurial")
-        os.symlink("%s/hg" % self.repository_path, "%s/.hg" % self.repository_path)
+        try:
+            os.symlink("%s/hg" % self.repository_path, "%s/.hg" % self.repository_path)
+        except OSError:
+            pass
         self.repos = MercurialRepository("file://%s" % self.repository_path)
         self.tmpdir = tempfile.mkdtemp()
         self.repos.checkout(self.tmpdir)
@@ -272,6 +340,7 @@ class TestMercurialModuleFunctions(unittest.TestCase):
         assert have_wc == True
 
 
+#@unittest.skipUnless(have_pysvn, "Could not import pysvn")
 class TestSubversionModuleFunctions(unittest.TestCase):
     pass
 
@@ -280,21 +349,26 @@ class TestPackageFunctions(unittest.TestCase):
     
     def setUp(self):
         self.hg_repos_path = os.path.abspath("../example_repositories/mercurial")
-        os.symlink("%s/hg" % self.hg_repos_path, "%s/.hg" % self.hg_repos_path)
+        try:
+            os.symlink("%s/hg" % self.hg_repos_path, "%s/.hg" % self.hg_repos_path)
+        except OSError:
+            pass
         self.basepath = os.path.abspath("%s/../example_repositories/" % os.getcwd())
         
     def tearDown(self):
         os.unlink("%s/.hg" % self.hg_repos_path)
     
     def test__get_repository__from_url(self):
-        repos = get_repository("file://%s/subversion" % self.basepath)
-        assert isinstance(repos, SubversionRepository)
+        if have_pysvn:
+            repos = get_repository("file://%s/subversion" % self.basepath)
+            assert isinstance(repos, SubversionRepository)
         repos = get_repository("file://%s/mercurial" % self.basepath)
         assert isinstance(repos, MercurialRepository), type(repos)
         
     def test__get_repository__from_invalid_url_should_raise_Exception(self):
         self.assertRaises(Exception, get_repository, "file:///tmp/")
 
+    @unittest.skipUnless(have_pysvn, "Could not import pysvn")
     def test__get_repository_from_working_copy(self):
         repos = SubversionRepository("file://%s/subversion" % self.basepath)
         tmpdir = tempfile.mkdtemp()
